//
// HyperoidMidlet - Hyperoids game ported to J2ME
//

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

/**
 * HyperoidMidlet contains the main game logic for Hyperoids.
 *
 * @see MIDlet
 *
 * @author Edward Hutchins
 * Version: 2.0M  Copyright (C) 2006 Hutchins Software
 * portions Copyright (C) 2000 Russell Marks
 * portions Copyright (C) 1996 Rasmus Ekman
 * Version: 1.1  Copyright (C) 1990,91 Hutchins Software
 * This software is licenced under the GNU General Public Licence
 * Please read the associated legal documentation
 */

public class HyperoidMidlet
    extends MIDlet
    implements CommandListener
{
    // A variable that holds the unique display
    private Display display = null;

    // The canvas
    private HyperoidCanvas canvas = null;

    // The MIDlet itself
    private static HyperoidMidlet self = null;

    // Called when the application starts, and when it is resumed.
    // We ignore the resume here and allocate data for our canvas
    // in the startApp method. This is generally very bad practice.
    protected void startApp() throws MIDletStateChangeException
    {
        // Set the self
        self = this;

        // Allocate
        display = Display.getDisplay( this );
        canvas = new HyperoidCanvas( display );

	// run at 30fps for now...
	canvas.init( 30 );

        // Add a quit command to the canvas
        // This command won't be seen, as we
        // are running in fullScreen mode
        // but it's always nice to have a quit command
        canvas.addCommand( new Command( "Quit", Command.EXIT, 1 ) );

        // Set the listener to be the MIDlet
        canvas.setCommandListener( this );

        // Start canvas
        display.setCurrent( canvas );

        canvas.start();
    }

    // Called when the game should pause, such as during a call
    protected void pauseApp()
    {
	if (canvas != null)
	{
	    canvas.stop();
	    while (!canvas.isDone())
		Thread.yield();
	    canvas = null;
	}
        System.gc();
    }

    // Called when the application should shut down
    protected void destroyApp( boolean unconditional ) throws MIDletStateChangeException
    {
	// in our cause pause should reclaim all resources
        pauseApp();
        // Method that shuts down the entire MIDlet
        notifyDestroyed();
    }

    // Listens to commands and processes
    public void commandAction( Command c, Displayable d )
    {
        // we actually don't care about command types here, either
	// the canvas is created (in which case we use the canvases
	// menu system), or it's not and we exit
	if (canvas != null)
	{
	    canvas.showRootMenu();
	}
	else
        {
	    try
	    {
		destroyApp( true );
	    }
	    catch (Exception e)
	    {
		System.out.println( "destroyApp exception (" + e.toString() + ")" );
	    }
        }
    }
    
    // Static method that quits our application by using the static field 'self'
    public static void die()
    {
	if (self != null)
	    self.notifyDestroyed();
    }

    // static method to print something to the console
    public static void conPrint( String s )
    {
	if (self.canvas != null)
	{
	    self.canvas.conPrint( s );
	}
	else System.out.println( s );
    }
}
